@@ -10,6 +10,8 @@
 #include <time.h>
 #include <dirent.h>
 
+#include <iostream>
+
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 
@@ -18,10 +20,7 @@
 #include <gnome-speech/gnome-speech.h>
 #endif
 
-#ifdef GNOME_LIBS
-#include <libgnomeui/libgnomeui.h>
-#include <libgnome/libgnome.h>
-#endif
+#include "mouse_input.h"
 
 #include "libdasher.h"
 
@@ -31,6 +30,11 @@
 #include "accessibility.h"
 #include "fileops.h"
 
+#ifdef GNOME_LIBS
+#include <libgnomeui/libgnomeui.h>
+#include <libgnome/libgnome.h>
+#endif
+
 #ifdef WITH_GPE
 #include "gpesettings_store.h"
 #else
@@ -53,11 +57,12 @@ GtkAccelGroup *dasher_accel;
 GtkWidget *dasher_menu_bar;
 GtkWidget *vpane;
 GtkFontSelectionDialog *dasher_fontselector, *edit_fontselector;
-GtkTreeSelection *alphselection, *colourselection;
-GtkWidget *alphabettreeview, *colourtreeview;
+GtkTreeSelection *alphselection, *colourselection, *lmsettingsselection;
+GtkWidget *alphabettreeview, *colourtreeview, *lmsettingstreeview;
 GtkWidget *preferences_window;
 GtkListStore *alph_list_store;
 GtkListStore *colour_list_store;
+GtkListStore *lmsettings_list_store;
 GladeXML *widgets;
 GtkWidget *open_filesel;
 GtkWidget *save_filesel;
@@ -73,6 +78,8 @@ GThread *trainthread;
 GAsyncQueue* trainqueue;
 ControlTree *controltree;
 
+CDasherMouseInput *pMouseInput;
+
 bool controlmodeon=false;
 bool keyboardmodeon=false;
 bool onedmode=false;
@@ -140,9 +147,12 @@ GPatternSpec *alphabetglob, *colourglob;
 GtkWidget *window;
 GtkWidget *file_selector;
 
-std::string dasherfont="DASHERFONT";
+std::string dasherfont=DASHERFONT;
 std::string editfont="Sans 10";
 
+int oldx;
+int oldy;
+
 double bitrate;
 
 void 
@@ -218,8 +228,10 @@ void update_colours()
       // want to support 2.0
       gchar ugly_path_hack[100];
       sprintf(ugly_path_hack,"%d",i);
-      gtk_tree_selection_select_path(colourselection,gtk_tree_path_new_from_string(ugly_path_hack));
-      gtk_tree_view_set_cursor(GTK_TREE_VIEW(colourtreeview),gtk_tree_path_new_from_string(ugly_path_hack),NULL,false);
+      GtkTreePath *path = gtk_tree_path_new_from_string(ugly_path_hack);
+      gtk_tree_selection_select_path(colourselection,path);
+      gtk_tree_view_set_cursor(GTK_TREE_VIEW(colourtreeview),path,NULL,false);
+      gtk_tree_path_free(path);
     }
   }
 }
@@ -244,6 +256,81 @@ extern "C" void colour_select(GtkTreeSelection *selection, gpointer data)
   }
 }
 
+extern "C" void lmsettings_edited_callback(GtkCellRendererText *cell, gchar *path_string, gchar *new_text, gpointer user_data) {
+
+  GtkTreeModel *model;
+  model = gtk_tree_view_get_model( GTK_TREE_VIEW(lmsettingstreeview) );
+
+  GtkTreeIter iter;
+  gtk_tree_model_get_iter_from_string( model, &iter, path_string );
+
+  gchar *gv;
+  gtk_tree_model_get( model, &iter, 0, &gv, -1 );
+
+  if( strcmp( gv, "LMMaxOrder" ) == 0 )
+    dasher_set_parameter_int( INT_LM_MAXORDER, atoi( new_text ) );
+  else if( strcmp( gv, "LMAlpha" ) == 0 )
+    dasher_set_parameter_int( INT_LM_ALPHA, atoi( new_text ) );
+  else if( strcmp( gv, "LMBeta" ) == 0 )
+    dasher_set_parameter_int( INT_LM_BETA, atoi( new_text ) );
+  else if( strcmp( gv, "LMExclusion" ) == 0 )
+    dasher_set_parameter_int( INT_LM_EXCLUSION, atoi( new_text ) );
+  else if( strcmp( gv, "LMUpdateExclusion" ) == 0 )
+    dasher_set_parameter_int( INT_LM_UPDATE_EXCLUSION, atoi( new_text ) );  
+  else if( strcmp( gv, "LMMixture" ) == 0 )
+  dasher_set_parameter_int( INT_LM_MIXTURE, atoi( new_text ) );
+
+}
+
+/* lmsettings has been disappeared from the glade interface!  --behdad
+extern "C" void
+generate_lm_options(GtkWidget *widget, gpointer user_data) {
+  GtkTreeViewColumn *column; 
+  GtkTreeIter lmsettingsiter;
+
+  lmsettingstreeview = glade_xml_get_widget(widgets,"lmsettingstree");  
+
+  gtk_widget_realize(lmsettingstreeview);
+
+  lmsettings_list_store = gtk_list_store_new(2,G_TYPE_STRING, G_TYPE_LONG);
+  gtk_tree_view_set_model(GTK_TREE_VIEW(lmsettingstreeview), GTK_TREE_MODEL(lmsettings_list_store));
+  lmsettingsselection = gtk_tree_view_get_selection (GTK_TREE_VIEW(lmsettingstreeview));
+  gtk_tree_selection_set_mode(GTK_TREE_SELECTION(lmsettingsselection),GTK_SELECTION_BROWSE);
+  column = gtk_tree_view_column_new_with_attributes ("Lmsettings",gtk_cell_renderer_text_new(),"text",0,NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(lmsettingstreeview),column);
+
+  GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
+  g_object_set(renderer, "editable", TRUE, NULL);
+
+  
+  column = gtk_tree_view_column_new_with_attributes ("Lmsettingsvals",GTK_CELL_RENDERER(renderer),"text",1,NULL);
+
+  g_signal_connect(renderer, "edited", (GCallback) lmsettings_edited_callback, NULL);
+
+  gtk_tree_view_append_column(GTK_TREE_VIEW(lmsettingstreeview),column);
+
+  // Clear the contents of the lmsettings list
+  gtk_list_store_clear( lmsettings_list_store );
+
+  gtk_list_store_append( lmsettings_list_store, &lmsettingsiter );
+  gtk_list_store_set(  lmsettings_list_store, &lmsettingsiter, 0, "LMMaxOrder", 1, 0, -1 );
+  gtk_list_store_append( lmsettings_list_store, &lmsettingsiter );
+  gtk_list_store_set(  lmsettings_list_store, &lmsettingsiter, 0, "LMAlpha", 1, 0, -1 );
+  gtk_list_store_append( lmsettings_list_store, &lmsettingsiter );
+  gtk_list_store_set(  lmsettings_list_store, &lmsettingsiter, 0, "LMBeta", 1, 0, -1 ); 
+  gtk_list_store_append( lmsettings_list_store, &lmsettingsiter );
+  gtk_list_store_set(  lmsettings_list_store, &lmsettingsiter, 0, "LMExclusion", 1, 0, -1 );
+  gtk_list_store_append( lmsettings_list_store, &lmsettingsiter );
+  gtk_list_store_set(  lmsettings_list_store, &lmsettingsiter, 0, "LMUpdateExclusion", 1, 0, -1 );
+  gtk_list_store_append( lmsettings_list_store, &lmsettingsiter );
+  gtk_list_store_set(  lmsettings_list_store, &lmsettingsiter, 0, "LMMixture", 1, 0, -1 );
+
+  //gtk_list_store_append( lmsettings_list_store, &lmsettingsiter );
+  //gtk_list_store_set(  lmsettings_list_store, &lmsettingsiter, 0, "LMBackoffConst", 1, 100, -1 );
+ 
+}
+*/
+
 extern "C" void 
 generate_preferences(GtkWidget *widget, gpointer user_data) { 
   // We need to populate the lists of alphabets and colours
@@ -282,7 +369,9 @@ generate_preferences(GtkWidget *widget, gpointer user_data) {
       gchar ugly_path_hack[100];
       sprintf(ugly_path_hack,"%d",i);
       gtk_tree_selection_select_iter(alphselection, &alphiter);
-      gtk_tree_view_set_cursor(GTK_TREE_VIEW(alphabettreeview),gtk_tree_path_new_from_string(ugly_path_hack),NULL,false);
+      GtkTreePath *path = gtk_tree_path_new_from_string(ugly_path_hack);
+      gtk_tree_view_set_cursor(GTK_TREE_VIEW(alphabettreeview),path,NULL,false);
+      gtk_tree_path_free(path);
     }
   }
   
@@ -316,11 +405,16 @@ generate_preferences(GtkWidget *widget, gpointer user_data) {
       gchar ugly_path_hack[100];
       sprintf(ugly_path_hack,"%d",i);
       gtk_tree_selection_select_iter(colourselection, &colouriter);
-      gtk_tree_view_set_cursor(GTK_TREE_VIEW(colourtreeview),gtk_tree_path_new_from_string(ugly_path_hack),NULL,false);
+      GtkTreePath *path = gtk_tree_path_new_from_string(ugly_path_hack);
+      gtk_tree_view_set_cursor(GTK_TREE_VIEW(colourtreeview),path,NULL,false);
+      gtk_tree_path_free(path);
     }
 
   }
 
+
+  
+
 }
 
 extern "C" void
@@ -855,30 +949,36 @@ timer_callback(gpointer data)
       }
     }
     gdk_window_get_pointer(the_canvas->window, &x, &y, NULL);
-    if (onedmode==true) {
-      // In one dimensional mode, we want to scale the vertical component so that it's possible
-      // for the amount of input to cover the entire canvas
-      float scalefactor;
-      float newy=y;
-      gdk_drawable_get_size(the_canvas->window, &dasherwidth, &dasherheight);
-      if (yscale==0) {
-	// For the magic value 0, we want the canvas size to reflect a full Y deflection
-	// otherwise the user can't access the entire range. 2 is actually a slight
-	// overestimate, but doing it properly would require thought and the benefit
-	// is probably insufficient.
-	scalefactor=2;
-      } else {
-	scalefactor=float(dasherheight)/float(yscale);
-      }
-      // Transform the real Y coordinate into a fudged Y coordinate
-      newy-=dasherheight/2;
-      newy=newy*scalefactor;
-      newy+=dasherheight/2;
-      y=int(newy);
-    } 
+
+    // This seems to be obsolete with the new dynamics, but check the excat behaviour
+
+//     if (onedmode==true) {
+//       // In one dimensional mode, we want to scale the vertical component so that it's possible
+//       // for the amount of input to cover the entire canvas
+//       float scalefactor;
+//       float newy=y;
+//       gdk_drawable_get_size(the_canvas->window, &dasherwidth, &dasherheight);
+//       if (yscale==0) {
+// 	// For the magic value 0, we want the canvas size to reflect a full Y deflection
+// 	// otherwise the user can't access the entire range. 2 is actually a slight
+// 	// overestimate, but doing it properly would require thought and the benefit
+// 	// is probably insufficient.
+// 	scalefactor=2;
+//       } else {
+// 	scalefactor=float(dasherheight)/float(yscale);
+//       }
+//       // Transform the real Y coordinate into a fudged Y coordinate
+//       newy-=dasherheight/2;
+//       newy=newy*scalefactor;
+//       newy+=dasherheight/2;
+//       y=int(newy);
+//     } 
     // And then provide the mouse position to the core. Of course, the core may then
     // do its own fudging.
-    dasher_tap_on( x, y, get_time() );
+
+    pMouseInput->SetCoordinates( x, y );
+
+    dasher_tap_on( 0, 0, get_time() );
   }
   
   else {
@@ -886,6 +986,8 @@ timer_callback(gpointer data)
     // reasons - start on mouse position, and to update the on-screen representation
     int x,y;
     gdk_window_get_pointer(the_canvas->window, &x, &y, NULL);
+
+     pMouseInput->SetCoordinates( x, y );
     
     if (onedmode==true) {
       float scalefactor;
@@ -901,8 +1003,12 @@ timer_callback(gpointer data)
       newy+=dasherheight/2;
       y=int(newy);
     } 
-    
-    dasher_draw_mouse_position(x,y);
+
+    if(( x != oldx ) || ( y != oldy )) // Only redraw if the mouse has actually moved
+      dasher_draw_mouse_position(x,y,0);
+
+    oldx = x;
+    oldy = y;
     
     if (mouseposstart==true) {
       // The user must hold the mouse pointer inside the red box, then the yellow box
@@ -1197,7 +1303,6 @@ slider_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 extern "C" gboolean
 button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-  GdkEventFocus *focusEvent = (GdkEventFocus *) g_malloc(sizeof(GdkEventFocus));
   gboolean *returnType;
 
 #ifdef WITH_GPE
@@ -1209,6 +1314,7 @@ button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer data)
     return FALSE;
 #endif
 
+  GdkEventFocus *focusEvent = (GdkEventFocus *) g_malloc(sizeof(GdkEventFocus));
   focusEvent->type = GDK_FOCUS_CHANGE;
   focusEvent->window = (GdkWindow *) the_canvas;
   focusEvent->send_event = FALSE;
@@ -1216,6 +1322,7 @@ button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer data)
 
   gtk_widget_grab_focus(GTK_WIDGET(the_canvas));
   g_signal_emit_by_name(GTK_OBJECT(the_canvas), "focus_in_event", GTK_WIDGET(the_canvas), focusEvent, NULL, &returnType);
+  g_free (focusEvent);
 
   // CJB,  2003-08.  If we have a selection, replace it with the new input.
   // This code is duplicated in key_press_event.
@@ -1278,7 +1385,7 @@ void interface_setup(GladeXML *xml) {
   speed_frame=glade_xml_get_widget(xml, "speed_frame");
   speed_hscale=GTK_SCALE(glade_xml_get_widget(xml, "speed_hscale"));
 
-  initialise_canvas(360,360);
+  initialise_canvas();
   initialise_edit();
 
   // interface specific preferences
@@ -1349,6 +1456,10 @@ void interface_setup(GladeXML *xml) {
   if (get_long_option_callback("Button9Y",&(buttons[9].y))==false) {
     buttons[9].y=0;
   }
+
+// lmsettings has been disappeared from the glade interface!  --behdad
+// generate_lm_options(NULL, NULL);
+
 }
 
 void
@@ -1440,6 +1551,9 @@ extern "C" void choose_filename() {
     getcwd(cwd,1024);
     filename = g_build_path("/",cwd,tbuffer,NULL);
     gtk_window_set_title(GTK_WINDOW(window), filename);
+
+    free(tbuffer);
+    free(cwd);
   } else {
     gtk_window_set_title(GTK_WINDOW(window), "Dasher");
     if (filename) {
@@ -1489,6 +1603,20 @@ extern "C" void orientation(GtkRadioButton *widget, gpointer user_data)
   dasher_redraw();
 }
 
+extern "C" void languagemodel(GtkRadioButton *widget, gpointer user_data)
+{
+  if (GTK_TOGGLE_BUTTON(widget)->active==TRUE) {
+    if (GTK_WIDGET(widget)==glade_xml_get_widget(widgets,"radiobutton6")) {
+      dasher_set_parameter_int( INT_LANGUAGEMODEL, 0 );
+    } else if (GTK_WIDGET(widget)==glade_xml_get_widget(widgets,"radiobutton7")) {
+      dasher_set_parameter_int( INT_LANGUAGEMODEL, 1 );
+    } else if (GTK_WIDGET(widget)==glade_xml_get_widget(widgets,"radiobutton8")) {
+      dasher_set_parameter_int( INT_LANGUAGEMODEL, 2 );
+    }
+  }
+  
+}
+
 extern "C" void set_dasher_fontsize(GtkWidget *widget, gpointer user_data)
 {
   if (GTK_CHECK_MENU_ITEM(widget)->active==TRUE) {
@@ -1640,7 +1768,7 @@ extern "C" void about_dasher(GtkWidget *widget, gpointer user_data)
   // In alphabetical order
   gchar *authors[] = {
     "Chris Ball",
-    "Phil Cowens",
+    "Phil Cowans",
     "Matthew Garrett",
     "David MacKay",
     "Iain Murray",
@@ -1752,10 +1880,10 @@ extern "C" void reset_fonts(GtkWidget *widget, gpointer user_data)
 {
   reset_edit_font();
   reset_dasher_font();
-  dasher_set_parameter_string( STRING_DASHERFONT, "DASHERFONT" );
+  dasher_set_parameter_string( STRING_DASHERFONT, DASHERFONT );
   dasher_set_parameter_string( STRING_EDITFONT, "Sans 10" );
   editfont="Sans 10";
-  dasherfont="DASHERFONT";
+  dasherfont=DASHERFONT;
 }
 
 extern "C" void speak(GtkWidget *widget, gpointer user_data)
@@ -1802,7 +1930,7 @@ void parameter_string_callback( string_param p, const char *value )
       set_canvas_font(value);
       dasherfont=value;
       if (dasherfont=="") {
-	dasherfont="DASHERFONT";
+	dasherfont=DASHERFONT;
       }
       break;
     case STRING_EDITFONT:
@@ -1829,7 +1957,8 @@ void parameter_double_callback( double_param p, double value )
 }
 
 void parameter_int_callback( int_param p, long int value )
-{
+{ 
+
   switch(p)
     {
     case INT_ORIENTATION:
@@ -1930,6 +2059,62 @@ void parameter_int_callback( int_param p, long int value )
 #endif
       dasher_redraw();
       break;
+
+/* lmsettings has been disappeared from the glade interface!  --behdad
+    case INT_LANGUAGEMODEL: 
+      switch( value ) {
+      case 0:
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(glade_xml_get_widget(widgets, "radiobutton6"))) != TRUE)
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(glade_xml_get_widget(widgets, "radiobutton6")), TRUE);
+	break;
+      case 1:
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(glade_xml_get_widget(widgets, "radiobutton7"))) != TRUE)
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(glade_xml_get_widget(widgets, "radiobutton7")), TRUE);
+	break;
+      case 2:
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(glade_xml_get_widget(widgets, "radiobutton8"))) != TRUE)
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(glade_xml_get_widget(widgets, "radiobutton8")), TRUE);
+	break;
+      }
+      break;
+
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+
+    case INT_LM_MAXORDER:
+      // FIXME - this seems a bit hacky
+      
+      model = gtk_tree_view_get_model( GTK_TREE_VIEW(lmsettingstreeview) );
+      gtk_tree_model_get_iter_from_string( model, &iter, "0" );
+      gtk_list_store_set( lmsettings_list_store, &iter, 1, value, -1 );
+      break;
+
+    case INT_LM_ALPHA: 
+      model = gtk_tree_view_get_model( GTK_TREE_VIEW(lmsettingstreeview) );
+      gtk_tree_model_get_iter_from_string( model, &iter, "1" );
+      gtk_list_store_set( lmsettings_list_store, &iter, 1, value, -1 );
+      break;
+    case INT_LM_BETA: 
+      model = gtk_tree_view_get_model( GTK_TREE_VIEW(lmsettingstreeview) );
+      gtk_tree_model_get_iter_from_string( model, &iter, "2" );
+      gtk_list_store_set( lmsettings_list_store, &iter, 1, value, -1 );
+      break;
+    case INT_LM_EXCLUSION: 
+      model = gtk_tree_view_get_model( GTK_TREE_VIEW(lmsettingstreeview) );
+      gtk_tree_model_get_iter_from_string( model, &iter, "3" );
+      gtk_list_store_set( lmsettings_list_store, &iter, 1, value, -1 );
+      break;
+    case INT_LM_UPDATE_EXCLUSION: 
+      model = gtk_tree_view_get_model( GTK_TREE_VIEW(lmsettingstreeview) );
+      gtk_tree_model_get_iter_from_string( model, &iter, "4" );
+      gtk_list_store_set( lmsettings_list_store, &iter, 1, value, -1 );
+      break;
+    case INT_LM_MIXTURE: 
+      model = gtk_tree_view_get_model( GTK_TREE_VIEW(lmsettingstreeview) );
+      gtk_tree_model_get_iter_from_string( model, &iter, "5" );
+      gtk_list_store_set( lmsettings_list_store, &iter, 1, value, -1 );
+      break;
+*/
     default:
       break;
     }
@@ -2102,6 +2287,7 @@ void scan_alphabet_files()
     }
   }
 
+  g_dir_close(directory);
   directory = g_dir_open(user_data_dir,0,NULL);
 
   while((filename=g_dir_read_name(directory))) {
@@ -2109,6 +2295,8 @@ void scan_alphabet_files()
       add_alphabet_filename(filename);
     }
   }
+
+  g_dir_close(directory);
 }
 
 void scan_colour_files()
@@ -2124,6 +2312,7 @@ void scan_colour_files()
     }
   }
 
+  g_dir_close(directory);
   directory = g_dir_open(user_data_dir,0,NULL);
 
   while((filename=g_dir_read_name(directory))) {
@@ -2131,6 +2320,8 @@ void scan_colour_files()
       add_colour_filename(filename);
     }
   }
+
+  g_dir_close(directory);
 }
 
 int alphabet_filter(const gchar* filename)
