@@ -186,7 +186,7 @@ void gtk2_edit_outputcontrol_callback(void* pointer, int data)
     break;
   case 31:
 	  if (pointer!=NULL) {
-		  wnck_window_activate((WnckWindow *)pointer, gtk_get_current_event_time());
+	    //		  wnck_window_activate((WnckWindow *)pointer, gtk_get_current_event_time());
 	  }
 	  break;
 #endif
@@ -593,7 +593,9 @@ void gtk2_get_new_context_callback( std::string &str, int max )
 
   gtk_text_iter_backward_chars(start, max);
 
-  str = std::string( gtk_text_buffer_get_text( the_text_buffer, start, end, FALSE ) );
+  char *s = gtk_text_buffer_get_text( the_text_buffer, start, end, FALSE );
+  str = std::string(s);
+  g_free (s);
 
   delete start;
   delete end;  
@@ -658,7 +660,9 @@ void speak_buffer()
 void set_editbox_font(std::string FontName)
 {
   if (FontName!="") {
-    gtk_widget_modify_font (the_text_view,pango_font_description_from_string(FontName.c_str()));
+    PangoFontDescription *pfd = pango_font_description_from_string(FontName.c_str());
+    gtk_widget_modify_font (the_text_view, pfd);
+    pango_font_description_free (pfd);
   }
 }
 
